import { A as AccessToken, D as DrupalMenuLinkContent, J as JsonApiWithLocaleOptions, a as JsonApiParams, b as JsonApiResource, c as DrupalTranslatedPath, L as Locale, B as BaseUrl, d as DrupalClientOptions, F as FetchOptions, e as JsonApiCreateResourceBody, f as JsonApiWithAuthOptions, g as DrupalFile, h as JsonApiCreateFileResourceBody, i as JsonApiUpdateResourceBody, j as JsonApiWithCacheOptions, P as PathPrefix, k as PathAlias, l as JsonApiResponse, m as PreviewOptions$1, n as DrupalView, o as DrupalClientAuthClientIdSecret, p as JsonApiError } from './types-5xvQrGBo.js';
export { z as DataCache, K as DrupalBlock, q as DrupalClientAuth, s as DrupalClientAuthAccessToken, r as DrupalClientAuthUsernamePassword, N as DrupalFileMeta, M as DrupalMedia, H as DrupalNode, I as DrupalParagraph, C as DrupalSearchApiFacet, O as DrupalTaxonomyTerm, Q as DrupalUser, y as Fetcher, G as GetResourcePreviewUrlOptions, v as JsonApiLinks, u as JsonApiOptions, w as JsonApiResourceBodyRelationship, E as JsonApiResourceWithPath, x as JsonApiSearchApiResponse, t as Logger, S as Serializer } from './types-5xvQrGBo.js';
import { GetStaticPathsContext, GetStaticPathsResult, GetStaticPropsContext, NextApiResponse, NextApiRequest } from 'next';
import * as jsona_lib_JsonaTypes from 'jsona/lib/JsonaTypes';

declare function getAccessToken(): Promise<AccessToken>;

declare function getMenu<T extends DrupalMenuLinkContent>(name: string, options?: {
    deserialize?: boolean;
    accessToken?: AccessToken;
} & JsonApiWithLocaleOptions): Promise<{
    items: T[];
    tree: T[];
}>;

declare function getPathsFromContext(types: string | string[], context: GetStaticPathsContext, options?: {
    params?: JsonApiParams;
    accessToken?: AccessToken;
}): Promise<GetStaticPathsResult["paths"]>;

declare function getResourceCollection<T = JsonApiResource[]>(type: string, options?: {
    deserialize?: boolean;
    accessToken?: AccessToken;
} & JsonApiWithLocaleOptions): Promise<T>;
declare function getResourceCollectionFromContext<T = JsonApiResource[]>(type: string, context: GetStaticPropsContext, options?: {
    deserialize?: boolean;
    params?: JsonApiParams;
}): Promise<T>;

interface PreviewOptions {
    errorMessages?: {
        secret?: string;
        slug?: string;
    };
}
declare function DrupalPreview(options?: PreviewOptions): (request: any, response: any) => Promise<void | NextApiResponse>;
declare function PreviewHandler(request?: NextApiRequest, response?: NextApiResponse, options?: PreviewOptions): Promise<void | NextApiResponse>;
type GetResourcePreviewUrlOptions = JsonApiWithLocaleOptions & {
    isVersionable?: boolean;
};
declare function getResourcePreviewUrl(slug: string, options?: GetResourcePreviewUrlOptions): Promise<any>;

declare function getResourceTypeFromContext(context: GetStaticPropsContext, options?: {
    accessToken?: AccessToken;
    prefix?: string;
}): Promise<string>;

declare function getResourceFromContext<T extends JsonApiResource>(type: string, context: GetStaticPropsContext, options?: {
    prefix?: string;
    deserialize?: boolean;
    params?: JsonApiParams;
    accessToken?: AccessToken;
    isVersionable?: boolean;
}): Promise<T>;
declare function getResourceByPath<T extends JsonApiResource>(path: string, options?: {
    accessToken?: AccessToken;
    deserialize?: boolean;
    isVersionable?: boolean;
} & JsonApiWithLocaleOptions): Promise<T>;
declare function getResource<T extends JsonApiResource>(type: string, uuid: string, options?: {
    accessToken?: AccessToken;
    deserialize?: boolean;
} & JsonApiWithLocaleOptions): Promise<T>;

declare function getSearchIndex<T = JsonApiResource[]>(name: string, options?: {
    deserialize?: boolean;
    accessToken?: AccessToken;
} & JsonApiWithLocaleOptions): Promise<T>;
declare function getSearchIndexFromContext<T = JsonApiResource[]>(name: string, context: GetStaticPropsContext, options?: {
    deserialize?: boolean;
    accessToken?: AccessToken;
} & JsonApiWithLocaleOptions): Promise<T>;

declare function getView<T>(name: string, options?: {
    deserialize?: boolean;
    accessToken?: AccessToken;
} & JsonApiWithLocaleOptions): Promise<{
    results: T;
    meta: Record<string, any>;
    links: {
        [key in "next" | "prev" | "self"]?: {
            href: "string";
        };
    };
}>;

declare function translatePath(path: string, options?: {
    accessToken?: AccessToken;
}): Promise<DrupalTranslatedPath>;
declare function translatePathFromContext(context: GetStaticPropsContext, options?: {
    accessToken?: AccessToken;
    prefix?: string;
}): Promise<DrupalTranslatedPath>;

declare function deserialize(body: any, options?: any): jsona_lib_JsonaTypes.TJsonaModel | jsona_lib_JsonaTypes.TJsonaModel[];
declare function getJsonApiPathForResourceType(type: string, locale?: Locale): Promise<string>;
declare function getJsonApiIndex(locale?: Locale, options?: {
    accessToken?: AccessToken;
}): Promise<{
    links: {
        [type: string]: {
            href: string;
        };
    };
}>;
declare function buildUrl(path: string, params?: string | Record<string, string> | URLSearchParams): URL;
declare function syncDrupalPreviewRoutes(path: any): void;

declare class DrupalClient {
    baseUrl: BaseUrl;
    debug: DrupalClientOptions["debug"];
    frontPage: DrupalClientOptions["frontPage"];
    private serializer;
    private cache;
    private throwJsonApiErrors?;
    private logger;
    private fetcher?;
    private _headers?;
    private _auth?;
    private _apiPrefix;
    private useDefaultResourceTypeEntry?;
    private _token?;
    private accessToken?;
    private accessTokenScope?;
    private tokenExpiresOn?;
    private withAuth?;
    private previewSecret?;
    private forceIframeSameSiteCookie?;
    /**
     * Instantiates a new DrupalClient.
     *
     * const client = new DrupalClient(baseUrl)
     *
     * @param {baseUrl} baseUrl The baseUrl of your Drupal site. Do not add the /jsonapi suffix.
     * @param {options} options Options for the client. See Experiment_DrupalClientOptions.
     */
    constructor(baseUrl: BaseUrl, options?: DrupalClientOptions);
    set apiPrefix(apiPrefix: DrupalClientOptions["apiPrefix"]);
    get apiPrefix(): DrupalClientOptions["apiPrefix"];
    set auth(auth: DrupalClientOptions["auth"]);
    set headers(value: DrupalClientOptions["headers"]);
    private set token(value);
    fetch(input: RequestInfo, init?: FetchOptions): Promise<Response>;
    createResource<T extends JsonApiResource>(type: string, body: JsonApiCreateResourceBody, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    createFileResource<T = DrupalFile>(type: string, body: JsonApiCreateFileResourceBody, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    updateResource<T extends JsonApiResource>(type: string, uuid: string, body: JsonApiUpdateResourceBody, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    deleteResource(type: string, uuid: string, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<boolean>;
    getResource<T extends JsonApiResource>(type: string, uuid: string, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions & JsonApiWithCacheOptions): Promise<T>;
    getResourceFromContext<T extends JsonApiResource>(input: string | DrupalTranslatedPath, context: GetStaticPropsContext, options?: {
        pathPrefix?: PathPrefix;
        isVersionable?: boolean;
    } & JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    getResourceByPath<T extends JsonApiResource>(path: string, options?: {
        isVersionable?: boolean;
    } & JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    getResourceCollection<T = JsonApiResource[]>(type: string, options?: {
        deserialize?: boolean;
    } & JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    getResourceCollectionFromContext<T = JsonApiResource[]>(type: string, context: GetStaticPropsContext, options?: {
        deserialize?: boolean;
    } & JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    getPathsFromContext: (types: string | string[], context: GetStaticPathsContext, options?: {
        params?: JsonApiParams;
        pathPrefix?: PathPrefix;
    } & JsonApiWithAuthOptions) => Promise<GetStaticPathsResult<{
        slug: string[];
    }>["paths"]>;
    getStaticPathsFromContext(types: string | string[], context: GetStaticPathsContext, options?: {
        params?: JsonApiParams;
        pathPrefix?: PathPrefix;
    } & JsonApiWithAuthOptions): Promise<GetStaticPathsResult<{
        slug: string[];
    }>["paths"]>;
    buildStaticPathsFromResources(resources: {
        path: PathAlias;
    }[], options?: {
        pathPrefix?: PathPrefix;
        locale?: Locale;
    }): {
        params: {
            slug: string[];
        };
    }[];
    buildStaticPathsParamsFromPaths(paths: string[], options?: {
        pathPrefix?: PathPrefix;
        locale?: Locale;
    }): {
        params: {
            slug: string[];
        };
    }[];
    translatePath(path: string, options?: JsonApiWithAuthOptions): Promise<DrupalTranslatedPath>;
    translatePathFromContext(context: GetStaticPropsContext, options?: {
        pathPrefix?: PathPrefix;
    } & JsonApiWithAuthOptions): Promise<DrupalTranslatedPath>;
    getPathFromContext(context: GetStaticPropsContext, options?: {
        pathPrefix?: PathPrefix;
    }): string;
    getIndex(locale?: Locale): Promise<JsonApiResponse>;
    getEntryForResourceType(type: string, locale?: Locale): Promise<string>;
    preview(request?: NextApiRequest, response?: NextApiResponse, options?: PreviewOptions$1): Promise<void | NextApiResponse>;
    getMenu<T = DrupalMenuLinkContent>(name: string, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions & JsonApiWithCacheOptions): Promise<{
        items: T[];
        tree: T[];
    }>;
    buildMenuTree(links: DrupalMenuLinkContent[], parent?: DrupalMenuLinkContent["id"]): any;
    getView<T = JsonApiResource>(name: string, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<DrupalView<T>>;
    getSearchIndex<T = JsonApiResource[]>(name: string, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    getSearchIndexFromContext<T = JsonApiResource[]>(name: string, context: GetStaticPropsContext, options?: JsonApiWithLocaleOptions & JsonApiWithAuthOptions): Promise<T>;
    buildUrl(path: string, params?: string | Record<string, string> | URLSearchParams | JsonApiParams): URL;
    getAccessToken(opts?: DrupalClientAuthClientIdSecret): Promise<AccessToken>;
    deserialize(body: any, options?: any): unknown;
    private getErrorsFromResponse;
    private formatJsonApiErrors;
    private _debug;
    private throwError;
    private handleJsonApiErrors;
    private getAuthFromContextAndOptions;
}

declare class JsonApiErrors extends Error {
    errors: JsonApiError[] | string;
    statusCode: number;
    constructor(errors: JsonApiError[], statusCode: number);
    private static formatMessage;
}

export { AccessToken, BaseUrl, DrupalClient, DrupalClientAuthClientIdSecret, DrupalClientOptions, DrupalFile, DrupalMenuLinkContent, DrupalPreview, DrupalTranslatedPath, DrupalView, FetchOptions, JsonApiCreateFileResourceBody, JsonApiCreateResourceBody, JsonApiError, JsonApiErrors, JsonApiParams, JsonApiResource, JsonApiResponse, JsonApiUpdateResourceBody, JsonApiWithAuthOptions, JsonApiWithCacheOptions, JsonApiWithLocaleOptions, Locale, PathAlias, PathPrefix, PreviewHandler, PreviewOptions$1 as PreviewOptions, buildUrl, deserialize, getAccessToken, getJsonApiIndex, getJsonApiPathForResourceType, getMenu, getPathsFromContext, getResource, getResourceByPath, getResourceCollection, getResourceCollectionFromContext, getResourceFromContext, getResourcePreviewUrl, getResourceTypeFromContext, getSearchIndex, getSearchIndexFromContext, getView, syncDrupalPreviewRoutes, translatePath, translatePathFromContext };
